# Copyright (c) OpenMMLab. All rights reserved.
# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import subprocess
import sys

import pytorch_sphinx_theme

sys.path.insert(0, os.path.abspath('..'))

# -- Project information -----------------------------------------------------

project = 'MMPose'
copyright = '2020-2021, OpenMMLab'
author = 'MMPose Authors'

# The full version, including alpha/beta/rc tags
version_file = '../mmpose/version.py'


def get_version():
    with open(version_file, 'r') as f:
        exec(compile(f.read(), version_file, 'exec'))
    return locals()['__version__']


release = get_version()

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc', 'sphinx.ext.napoleon', 'sphinx.ext.viewcode',
    'sphinx_markdown_tables', 'sphinx_copybutton', 'myst_parser'
]

autodoc_mock_imports = ['json_tricks', 'mmpose.version']

# Ignore >>> when copying code
copybutton_prompt_text = r'>>> |\.\.\. '
copybutton_prompt_is_regexp = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# -- Options for HTML output -------------------------------------------------
source_suffix = {
    '.rst': 'restructuredtext',
    '.md': 'markdown',
}

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'pytorch_sphinx_theme'
html_theme_path = [pytorch_sphinx_theme.get_html_theme_path()]
html_theme_options = {
    'menu': [
        {
            'name':
            'Tutorial',
            'url':
            'https://colab.research.google.com/github/'
            'open-mmlab/mmpose/blob/master/demo/MMPose_Tutorial.ipynb'
        },
        {
            'name': 'GitHub',
            'url': 'https://github.com/open-mmlab/mmpose'
        },
        {
            'name':
            'Projects',
            'children': [{
                'name':
                'MMCV',
                'url':
                'https://mmcv.readthedocs.io/en/latest/',
                'description':
                'Foundational library for computer vision'
            }, {
                'name':
                'MMDetection',
                'url':
                'https://mmdetection.readthedocs.io/en/latest/',
                'description':
                'Object detection toolbox and benchmark'
            }, {
                'name':
                'MMAction2',
                'url':
                'https://mmaction2.readthedocs.io/en/latest/',
                'description':
                'Action understanding toolbox and benchmark'
            }, {
                'name':
                'MMClassification',
                'url':
                'https://mmclassification.readthedocs.io/en/latest/',
                'description':
                'Image classification toolbox and benchmark'
            }, {
                'name':
                'MMSegmentation',
                'url':
                'https://mmsegmentation.readthedocs.io/en/latest/',
                'description':
                'Semantic segmentation toolbox and benchmark'
            }, {
                'name': 'MMDetection3D',
                'url': 'https://mmdetection3d.readthedocs.io/en/latest/',
                'description': 'General 3D object detection platform'
            }, {
                'name': 'MMEditing',
                'url': 'https://mmediting.readthedocs.io/en/latest/',
                'description': 'Image and video editing toolbox'
            }, {
                'name':
                'MMOCR',
                'url':
                'https://mmocr.readthedocs.io/en/latest/',
                'description':
                'Text detection, recognition and understanding toolbox'
            }, {
                'name':
                'MMTracking',
                'url':
                'https://mmtracking.readthedocs.io/en/latest/',
                'description':
                'Video perception toolbox and benchmark'
            }, {
                'name': 'MMGeneration',
                'url': 'https://mmgeneration.readthedocs.io/en/latest/',
                'description': 'Generative model toolbox'
            }, {
                'name': 'MMFlow',
                'url': 'https://mmflow.readthedocs.io/en/latest/',
                'description': 'Optical flow toolbox and benchmark'
            }, {
                'name':
                'MMFewShot',
                'url':
                'https://mmfewshot.readthedocs.io/en/latest/',
                'description':
                'FewShot learning toolbox and benchmark'
            }, {
                'name':
                'MMHuman3D',
                'url':
                'https://mmhuman3d.readthedocs.io/en/latest/',
                'description':
                '3D human parametric model toolbox and benchmark.'
            }]
        },
        {
            'name':
            'OpenMMLab',
            'children': [{
                'name': 'Homepage',
                'url': 'https://openmmlab.com/'
            }, {
                'name': 'GitHub',
                'url': 'https://github.com/open-mmlab/'
            }, {
                'name': 'Twitter',
                'url': 'https://twitter.com/OpenMMLab'
            }, {
                'name': 'Zhihu',
                'url': 'https://zhihu.com/people/openmmlab'
            }]
        },
    ]
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".

language = 'en'

html_static_path = ['_static']
html_css_files = ['css/readthedocs.css']

# Enable ::: for my_st
myst_enable_extensions = ['colon_fence']

master_doc = 'index'


def builder_inited_handler(app):
    subprocess.run(['./collect.py'])
    subprocess.run(['./merge_docs.sh'])
    subprocess.run(['./stats.py'])


def setup(app):
    app.connect('builder-inited', builder_inited_handler)
